@@ -1,59 +1,68 @@
 from __future__ import annotations
 
 import logging
-import re
 import time
+import warnings
 from abc import ABCMeta, abstractmethod
-from functools import wraps
 from types import TracebackType
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Iterator,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, Union
+from urllib.parse import urljoin
+
+from httpx import (
+    URL,
+    USE_CLIENT_DEFAULT,
+    Headers,
+    Response,
+    TimeoutException,
+    codes,
 )
 
-from httpx import URL
-from httpx import AsyncClient as HttpxAsyncClient
-from httpx import Response, codes
-
-from firebolt.async_db.util import ENGINE_STATUS_RUNNING
-from firebolt.client.client import AsyncClientV1, AsyncClientV2
-from firebolt.common._types import (
-    ColType,
-    Column,
-    ParameterType,
-    RawColType,
-    SetParameter,
-    split_format_sql,
-)
-from firebolt.common.base_cursor import (
+from firebolt.client.client import AsyncClient, AsyncClientV1, AsyncClientV2
+from firebolt.common._types import ColType, ParameterType, SetParameter
+from firebolt.common.constants import (
     JSON_OUTPUT_FORMAT,
-    BaseCursor,
+    RESET_SESSION_HEADER,
+    UPDATE_ENDPOINT_HEADER,
+    UPDATE_PARAMETERS_HEADER,
     CursorState,
-    QueryStatus,
-    Statistics,
+)
+from firebolt.common.cursor.base_cursor import (
+    BaseCursor,
+    _parse_update_endpoint,
+    _parse_update_parameters,
+    _raise_if_internal_set_parameter,
+)
+from firebolt.common.cursor.decorators import (
+    async_not_allowed,
     check_not_closed,
     check_query_executed,
 )
+from firebolt.common.cursor.statement_planners import (
+    ExecutionPlan,
+    StatementPlannerFactory,
+)
+from firebolt.common.row_set.asynchronous.base import BaseAsyncRowSet
+from firebolt.common.row_set.asynchronous.in_memory import InMemoryAsyncRowSet
+from firebolt.common.row_set.asynchronous.streaming import StreamingAsyncRowSet
+from firebolt.common.statement_formatter import create_statement_formatter
 from firebolt.utils.exception import (
-    AsyncExecutionUnavailableError,
     EngineNotRunningError,
     FireboltDatabaseError,
-    FireboltEngineError,
+    FireboltError,
     OperationalError,
     ProgrammingError,
+    QueryTimeoutError,
+    V1NotSupportedError,
 )
+from firebolt.utils.timeout_controller import TimeoutController
 from firebolt.utils.urls import DATABASES_URL, ENGINES_URL
 
 if TYPE_CHECKING:
     from firebolt.async_db.connection import Connection
 
-from firebolt.utils.util import Timer, _print_error_body
+from firebolt.utils.async_util import anext, async_islice
+from firebolt.utils.cache import ConnectionInfo, DatabaseInfo, EngineInfo
+from firebolt.utils.util import Timer, raise_error_from_response
 
 logger = logging.getLogger(__name__)
 
@@ -75,162 +84,274 @@ class Cursor(BaseCursor, metaclass=ABCMeta):
     def __init__(
         self,
         *args: Any,
-        client: HttpxAsyncClient,
+        client: AsyncClient,
         connection: Connection,
         **kwargs: Any,
     ) -> None:
         super().__init__(*args, **kwargs)
-        self._client = client
         self.connection = connection
-        if connection.database:
-            self.database = connection.database
+        self._client: AsyncClient = client
+        self.engine_url = connection.engine_url
+        self._row_set: Optional[BaseAsyncRowSet] = None
+        if connection.init_parameters:
+            self._update_set_parameters(connection.init_parameters)
 
-    @property
-    def database(self) -> Optional[str]:
-        return self.parameters.get("database")
-
-    @database.setter
-    def database(self, database: str) -> None:
-        self.parameters["database"] = database
-
-    @abstractmethod
     async def _api_request(
         self,
         query: str = "",
         parameters: Optional[dict[str, Any]] = None,
         path: str = "",
         use_set_parameters: bool = True,
+        timeout: Optional[float] = None,
     ) -> Response:
-        ...
+        """
+        Query API, return Response object.
+
+        Args:
+            query (str): SQL query
+            parameters (Optional[Sequence[ParameterType]]): A sequence of substitution
+                parameters. Used to replace '?' placeholders inside a query with
+                actual values. Note: In order to "output_format" dict value, it
+                    must be an empty string. If no value not specified,
+                    JSON_OUTPUT_FORMAT will be used.
+            path (str): endpoint suffix, for example "cancel" or "status"
+            use_set_parameters: Optional[bool]: Some queries will fail if additional
+                set parameters are sent. Setting this to False will allow
+                self._set_parameters to be ignored.
+            timeout (Optional[float]): Request execution timeout in seconds
+        """
+        parameters = parameters or {}
+        if use_set_parameters:
+            parameters = {**(self._set_parameters or {}), **parameters}
+        if self.parameters:
+            parameters = {**self.parameters, **parameters}
+        try:
+            req = self._client.build_request(
+                url=urljoin(self.engine_url.rstrip("/") + "/", path or ""),
+                method="POST",
+                params=parameters,
+                content=query,
+                timeout=timeout if timeout is not None else USE_CLIENT_DEFAULT,
+            )
+            return await self._client.send(req, stream=True)
+        except TimeoutException:
+            raise QueryTimeoutError()
 
     async def _raise_if_error(self, resp: Response) -> None:
         """Raise a proper error if any"""
-        if resp.status_code == codes.INTERNAL_SERVER_ERROR:
-            raise OperationalError(
-                f"Error executing query:\n{resp.read().decode('utf-8')}"
-            )
-        if resp.status_code == codes.FORBIDDEN:
-            if self.database and not await self.is_db_available(self.database):
-                raise FireboltDatabaseError(f"Database {self.database} does not exist")
-            raise ProgrammingError(resp.read().decode("utf-8"))
-        if (
-            resp.status_code == codes.SERVICE_UNAVAILABLE
-            or resp.status_code == codes.NOT_FOUND
-        ) and not await self.is_engine_running(self.connection.engine_url):
-            raise EngineNotRunningError(
-                f"Firebolt engine {self.connection.engine_url} "
-                "needs to be running to run queries against it."
-            )
-        _print_error_body(resp)
-        resp.raise_for_status()
+        if codes.is_error(resp.status_code):
+            await resp.aread()
+            if resp.status_code == codes.INTERNAL_SERVER_ERROR:
+                raise OperationalError(f"Error executing query:\n{resp.text}")
+            if resp.status_code == codes.FORBIDDEN:
+                if self.database and not await self.is_db_available(self.database):
+                    raise FireboltDatabaseError(
+                        f"Database {self.database} does not exist"
+                    )
+                raise ProgrammingError(resp.text)
+            if (
+                resp.status_code == codes.SERVICE_UNAVAILABLE
+                or resp.status_code == codes.NOT_FOUND
+            ) and not await self.is_engine_running(self.engine_url):
+                raise EngineNotRunningError(
+                    f"Firebolt engine {self.engine_url} "
+                    "needs to be running to run queries against it."
+                )
+            raise_error_from_response(resp)
 
-    async def _validate_set_parameter(self, parameter: SetParameter) -> None:
+    async def _validate_set_parameter(
+        self, parameter: SetParameter, timeout: Optional[float]
+    ) -> None:
         """Validate parameter by executing simple query with it."""
-        if parameter.name == "async_execution":
-            raise AsyncExecutionUnavailableError(
-                "It is not possible to set async_execution using a SET command. "
-                "Instead, pass it as an argument to the execute() or "
-                "executemany() function."
-            )
-        resp = await self._api_request("select 1", {parameter.name: parameter.value})
+        _raise_if_internal_set_parameter(parameter)
+        resp = await self._api_request(
+            "select 1", {parameter.name: parameter.value}, timeout=timeout
+        )
         # Handle invalid set parameter
         if resp.status_code == codes.BAD_REQUEST:
+            await resp.aread()
             raise OperationalError(resp.text)
         await self._raise_if_error(resp)
 
         # set parameter passed validation
         self._set_parameters[parameter.name] = parameter.value
 
+        # append empty result set
+        await self._append_row_set_from_response(None)
+
+    async def _parse_response_headers(self, headers: Headers) -> None:
+        if headers.get(UPDATE_ENDPOINT_HEADER):
+            endpoint, params = _parse_update_endpoint(
+                headers.get(UPDATE_ENDPOINT_HEADER)
+            )
+            self._update_set_parameters(params)
+            self.engine_url = endpoint
+
+        if headers.get(RESET_SESSION_HEADER):
+            self.flush_parameters()
+
+        if headers.get(UPDATE_PARAMETERS_HEADER):
+            param_dict = _parse_update_parameters(headers.get(UPDATE_PARAMETERS_HEADER))
+            self._update_set_parameters(param_dict)
+
+    async def _close_rowset_and_reset(self) -> None:
+        """Reset cursor state."""
+        if self._row_set is not None:
+            await self._row_set.aclose()
+        super()._reset()
+
+    @abstractmethod
+    async def execute_async(
+        self,
+        query: str,
+        parameters: Optional[Sequence[ParameterType]] = None,
+        skip_parsing: bool = False,
+    ) -> int:
+        """Execute a database query without maintaining a connection."""
+        ...
+
     async def _do_execute(
         self,
         raw_query: str,
         parameters: Sequence[Sequence[ParameterType]],
         skip_parsing: bool = False,
-        async_execution: Optional[bool] = False,
+        timeout: Optional[float] = None,
+        async_execution: bool = False,
+        streaming: bool = False,
+        bulk_insert: bool = False,
     ) -> None:
-        self._reset()
-        # Allow users to manually skip parsing for performance improvement.
-        queries: List[Union[SetParameter, str]] = (
-            [raw_query] if skip_parsing else split_format_sql(raw_query, parameters)
-        )
+        await self._close_rowset_and_reset()
+        self._row_set = StreamingAsyncRowSet() if streaming else InMemoryAsyncRowSet()
+
+        # Import paramstyle from module level
+        from firebolt.async_db import paramstyle
+
         try:
-            for query in queries:
-                start_time = time.time()
-                # Our CREATE EXTERNAL TABLE queries currently require credentials,
-                # so we will skip logging those queries.
-                # https://docs.firebolt.io/sql-reference/commands/create-external-table.html
-                if isinstance(query, SetParameter) or not re.search(
-                    "aws_key_id|credentials", query, flags=re.IGNORECASE
-                ):
-                    logger.debug(f"Running query: {query}")
-
-                # Define type for mypy
-                row_set: Tuple[
-                    int,
-                    Optional[List[Column]],
-                    Optional[Statistics],
-                    Optional[List[List[RawColType]]],
-                ] = (-1, None, None, None)
-                if isinstance(query, SetParameter):
-                    await self._validate_set_parameter(query)
-                elif async_execution:
-                    self._validate_server_side_async_settings(
-                        parameters,
-                        queries,
-                        skip_parsing,
-                        async_execution,
-                    )
-
-                    with Timer(
-                        f"[PERFORMANCE] Running query {query[:50]} "
-                        f"{'... ' if len(query) > 50 else ''}"
-                    ):
-                        response = await self._api_request(
-                            query,
-                            {
-                                "async_execution": 1,
-                                "output_format": JSON_OUTPUT_FORMAT,
-                            },
-                        )
-
-                    await self._raise_if_error(response)
-                    if response.headers.get("content-length", "") == "0":
-                        raise OperationalError("No response to asynchronous query.")
-                    resp = response.json()
-                    if "query_id" not in resp or resp["query_id"] == "":
-                        raise OperationalError(
-                            "Invalid response to asynchronous query: missing query_id."
-                        )
-                    self._query_id = resp["query_id"]
-                else:
-                    resp = await self._api_request(
-                        query, {"output_format": JSON_OUTPUT_FORMAT}
-                    )
-                    await self._raise_if_error(resp)
-                    # get parameters from response
-                    self._parse_response_headers(resp.headers)
-                    row_set = self._row_set_from_response(resp)
-
-                self._append_row_set(row_set)
-
-                logger.info(
-                    f"Query fetched {self.rowcount} rows in"
-                    f" {time.time() - start_time} seconds."
-                )
+            statement_planner = StatementPlannerFactory.create_planner(
+                paramstyle, self._formatter
+            )
 
+            plan = statement_planner.create_execution_plan(
+                raw_query,
+                parameters,
+                skip_parsing,
+                async_execution,
+                streaming,
+                bulk_insert,
+            )
+            await self._execute_plan(plan, timeout)
             self._state = CursorState.DONE
-
         except Exception:
             self._state = CursorState.ERROR
             raise
 
+    async def _execute_plan(
+        self,
+        plan: ExecutionPlan,
+        timeout: Optional[float],
+    ) -> None:
+        """Execute an execution plan."""
+        timeout_controller = TimeoutController(timeout)
+
+        for query in plan.queries:
+            if isinstance(query, SetParameter):
+                if plan.async_execution:
+                    raise FireboltError(
+                        "Server side async does not support set statements, "
+                        "please use execute to set this parameter"
+                    )
+                await self._validate_set_parameter(
+                    query, timeout_controller.remaining()
+                )
+            else:
+                # Regular query execution
+                await self._execute_single_query(
+                    query,
+                    plan.query_params,
+                    timeout_controller,
+                    plan.async_execution,
+                    plan.streaming,
+                )
+
+    async def _execute_single_query(
+        self,
+        query: str,
+        query_params: Optional[Dict[str, Any]],
+        timeout_controller: TimeoutController,
+        async_execution: bool,
+        streaming: bool,
+    ) -> None:
+        """Execute a single query."""
+        start_time = time.time()
+        Cursor._log_query(query)
+        timeout_controller.raise_if_timeout()
+
+        final_params = query_params or {}
+
+        resp = await self._api_request(
+            query,
+            final_params,
+            timeout=timeout_controller.remaining(),
+        )
+        await self._raise_if_error(resp)
+
+        if async_execution:
+            await resp.aread()
+            self._parse_async_response(resp)
+        else:
+            await self._parse_response_headers(resp.headers)
+            await self._append_row_set_from_response(resp)
+
+        if not async_execution:
+            logger.info(
+                f"Query fetched {self.rowcount} rows in"
+                f" {time.time() - start_time} seconds."
+            )
+        else:
+            logger.info("Query submitted for async execution.")
+
+    async def use_database(self, database: str, cache: bool = True) -> None:
+        """Switch the current database context with caching."""
+        if cache:
+            cache_record = self.get_cache_record()
+            cache_record = (
+                cache_record if cache_record else ConnectionInfo(id=self.connection.id)
+            )
+            if cache_record.databases.get(database):
+                # If database is cached, use it
+                self.database = database
+            else:
+                await self.execute(f'USE DATABASE "{database}"')
+                cache_record.databases[database] = DatabaseInfo(database)
+                self.set_cache_record(cache_record)
+        else:
+            await self.execute(f'USE DATABASE "{database}"')
+
+    async def use_engine(self, engine: str, cache: bool = True) -> None:
+        """Switch the current engine context with caching."""
+        if cache:
+            cache_obj = self.get_cache_record()
+            cache_obj = (
+                cache_obj if cache_obj else ConnectionInfo(id=self.connection.id)
+            )
+            if cache_obj.engines.get(engine):
+                # If engine is cached, use it
+                self.engine_url = cache_obj.engines[engine].url
+                self._update_set_parameters(cache_obj.engines[engine].params)
+            else:
+                await self.execute(f'USE ENGINE "{engine}"')
+                cache_obj.engines[engine] = EngineInfo(self.engine_url, self.parameters)
+                self.set_cache_record(cache_obj)
+        else:
+            await self.execute(f'USE ENGINE "{engine}"')
+
     @check_not_closed
     async def execute(
         self,
         query: str,
         parameters: Optional[Sequence[ParameterType]] = None,
         skip_parsing: bool = False,
-        async_execution: Optional[bool] = False,
+        timeout_seconds: Optional[float] = None,
     ) -> Union[int, str]:
         """Prepare and execute a database query.
 
@@ -254,21 +375,24 @@ class Cursor(BaseCursor, metaclass=ABCMeta):
             skip_parsing (bool): Flag to disable query parsing. This will
                 disable parameterized, multi-statement and SET queries,
                 while improving performance
-            async_execution (bool): flag to determine if query should be asynchronous
+            timeout_seconds (Optional[float]): Query execution timeout in seconds
 
         Returns:
             int: Query row count.
         """
         params_list = [parameters] if parameters else []
-        await self._do_execute(query, params_list, skip_parsing, async_execution)
-        return self.query_id if async_execution else self.rowcount
+        await self._do_execute(
+            query, params_list, skip_parsing, timeout=timeout_seconds
+        )
+        return self.rowcount
 
     @check_not_closed
     async def executemany(
         self,
         query: str,
         parameters_seq: Sequence[Sequence[ParameterType]],
-        async_execution: Optional[bool] = False,
+        timeout_seconds: Optional[float] = None,
+        bulk_insert: bool = False,
     ) -> Union[int, str]:
         """Prepare and execute a database query.
 
@@ -286,6 +410,9 @@ class Cursor(BaseCursor, metaclass=ABCMeta):
                 `SET param=value` statement before it. All parameters are stored in
                 cursor object until it's closed. They can also be removed with
                 `flush_parameters` method call.
+            Bulk insert: When bulk_insert=True, multiple INSERT queries are
+                concatenated and sent as a single batch for improved performance.
+                Only supported for INSERT statements.
 
         Args:
             query (str): SQL query to execute.
@@ -293,46 +420,115 @@ class Cursor(BaseCursor, metaclass=ABCMeta):
                substitution parameter sets. Used to replace '?' placeholders inside a
                query with actual values from each set in a sequence. Resulting queries
                for each subset are executed sequentially.
-            async_execution (bool): flag to determine if query should be asynchronous
+            timeout_seconds (Optional[float]): Query execution timeout in seconds.
+            bulk_insert (bool): When True, concatenates multiple INSERT queries
+               into a single batch request. Only supported for INSERT statements.
 
         Returns:
-            int|str: Query row count for synchronous execution of queries,
-            query ID string for asynchronous execution.
+            int: Query row count.
         """
-        await self._do_execute(query, parameters_seq, async_execution=async_execution)
-        if async_execution:
-            return self.query_id
-        else:
-            return self.rowcount
+        await self._do_execute(
+            query, parameters_seq, timeout=timeout_seconds, bulk_insert=bulk_insert
+        )
+        return self.rowcount
 
     @check_not_closed
-    async def get_status(self, query_id: str) -> QueryStatus:
-        """Get status of a server-side async query. Return the state of the query."""
-        try:
-            resp = await self._api_request(
-                # output_format must be empty for status to work correctly.
-                # And set parameters will cause 400 errors.
-                parameters={"query_id": query_id},
-                path="status",
-                use_set_parameters=False,
-            )
-            if resp.status_code == codes.BAD_REQUEST:
-                raise OperationalError(
-                    f"Asynchronous query {query_id} status check failed: "
-                    f"{resp.status_code}."
-                )
-            resp_json = resp.json()
-            if "status" not in resp_json:
-                raise OperationalError(
-                    "Invalid response to asynchronous query: missing status."
-                )
-        except Exception:
-            self._state = CursorState.ERROR
-            raise
-        # Remember that query_id might be empty.
-        if resp_json["status"] == "":
-            return QueryStatus.NOT_READY
-        return QueryStatus[resp_json["status"]]
+    async def execute_stream(
+        self,
+        query: str,
+        parameters: Optional[Sequence[ParameterType]] = None,
+        skip_parsing: bool = False,
+    ) -> None:
+        """Prepare and execute a database query, with streaming results.
+
+        Supported features:
+            Parameterized queries: Placeholder characters ('?') are substituted
+                with values provided in `parameters`. Values are formatted to
+                be properly recognized by database and to exclude SQL injection.
+            Multi-statement queries: Multiple statements, provided in a single query
+                and separated by semicolon, are executed separately and sequentially.
+                To switch to next statement result, use `nextset` method.
+            SET statements: To provide additional query execution parameters, execute
+                `SET param=value` statement before it. All parameters are stored in
+                cursor object until it's closed. They can also be removed with
+                `flush_parameters` method call.
+
+        Args:
+            query (str): SQL query to execute.
+            parameters (Optional[Sequence[ParameterType]]): Substitution parameters.
+                Used to replace '?' placeholders inside a query with actual values.
+            skip_parsing (bool): Flag to disable query parsing. This will
+                disable parameterized, multi-statement and SET queries,
+                while improving performance
+        """
+        params_list = [parameters] if parameters else []
+        await self._do_execute(query, params_list, skip_parsing, streaming=True)
+
+    async def _append_row_set_from_response(
+        self,
+        response: Optional[Response],
+    ) -> None:
+        """Store information about executed query."""
+        if self._row_set is None:
+            raise OperationalError("Row set is not initialized.")
+        if response is None:
+            self._row_set.append_empty_response()
+        else:
+            await self._row_set.append_response(response)
+
+    _performance_log_message = (
+        "[PERFORMANCE] Parsing query output into native Python types"
+    )
+
+    @check_not_closed
+    @async_not_allowed
+    @check_query_executed
+    async def fetchone(self) -> Optional[List[ColType]]:
+        """Fetch the next row of a query result set."""
+        assert self._row_set is not None
+        with Timer(self._performance_log_message):
+            return await anext(self._row_set, None)
+
+    @check_not_closed
+    @async_not_allowed
+    @check_query_executed
+    async def fetchmany(self, size: Optional[int] = None) -> List[List[ColType]]:
+        """
+        Fetch the next set of rows of a query result;
+        cursor.arraysize is default size.
+        """
+        assert self._row_set is not None
+        size = size if size is not None else self.arraysize
+        with Timer(self._performance_log_message):
+            return await async_islice(self._row_set, size)
+
+    @check_not_closed
+    @async_not_allowed
+    @check_query_executed
+    async def fetchall(self) -> List[List[ColType]]:
+        """Fetch all remaining rows of a query result."""
+        assert self._row_set is not None
+        with Timer(self._performance_log_message):
+            return [it async for it in self._row_set]
+
+    @check_not_closed
+    @async_not_allowed
+    @check_query_executed
+    async def nextset(self) -> bool:
+        """
+        Skip to the next available set, discarding any remaining rows
+        from the current set.
+
+        Returns:
+            bool: True if there is a next result set, False otherwise
+        """
+        assert self._row_set is not None
+        return await self._row_set.nextset()
+
+    async def aclose(self) -> None:
+        super().close()
+        if self._row_set is not None:
+            await self._row_set.aclose()
 
     @abstractmethod
     async def is_db_available(self, database: str) -> bool:
@@ -344,72 +540,41 @@ class Cursor(BaseCursor, metaclass=ABCMeta):
         """Verify that the engine is running."""
         ...
 
-    @wraps(BaseCursor.fetchone)
-    async def fetchone(self) -> Optional[List[ColType]]:
-        """Fetch the next row of a query result set."""
-        return super().fetchone()
-
-    @wraps(BaseCursor.fetchmany)
-    async def fetchmany(self, size: Optional[int] = None) -> List[List[ColType]]:
-        """
-        Fetch the next set of rows of a query result;
-        size is cursor.arraysize by default.
-        """
-        return super().fetchmany(size)
-
-    @wraps(BaseCursor.fetchall)
-    async def fetchall(self) -> List[List[ColType]]:
-        """Fetch all remaining rows of a query result."""
-        return super().fetchall()
-
-    @wraps(BaseCursor.nextset)
-    async def nextset(self) -> None:
-        return super().nextset()
-
-    @check_not_closed
-    async def cancel(self, query_id: str) -> None:
-        """Cancel a server-side async query."""
-        await self._api_request(
-            parameters={"query_id": query_id},
-            path="cancel",
-            use_set_parameters=False,
-        )
-
     # Iteration support
     @check_not_closed
+    @async_not_allowed
     @check_query_executed
     def __aiter__(self) -> Cursor:
         return self
 
-    # TODO: figure out how to implement __aenter__ and __await__
     @check_not_closed
-    def __aenter__(self) -> Cursor:
+    async def __aenter__(self) -> Cursor:
         return self
 
+    async def __aexit__(
+        self, exc_type: type, exc_val: Exception, exc_tb: TracebackType
+    ) -> None:
+        await self.aclose()
+
+    @check_not_closed
+    @async_not_allowed
+    @check_query_executed
+    async def __anext__(self) -> List[ColType]:
+        assert self._row_set is not None
+        return await self._row_set.__anext__()
+
     @check_not_closed
     def __enter__(self) -> Cursor:
+        warnings.warn(
+            "Using __enter__ is deprecated, use 'async with' instead",
+            DeprecationWarning,
+        )
         return self
 
     def __exit__(
         self, exc_type: type, exc_val: Exception, exc_tb: TracebackType
     ) -> None:
-        self.close()
-
-    def __await__(self) -> Iterator:
-        yield None
-
-    async def __aexit__(
-        self, exc_type: type, exc_val: Exception, exc_tb: TracebackType
-    ) -> None:
-        self.close()
-
-    @check_not_closed
-    @check_query_executed
-    async def __anext__(self) -> List[ColType]:
-        row = await self.fetchone()
-        if row is None:
-            raise StopAsyncIteration
-        return row
+        return None
 
 
 class CursorV2(Cursor):
@@ -421,44 +586,54 @@ class CursorV2(Cursor):
         **kwargs: Any,
     ) -> None:
         assert isinstance(client, AsyncClientV2)
-        super().__init__(*args, client=client, connection=connection, **kwargs)
+        super().__init__(
+            *args,
+            client=client,
+            connection=connection,
+            formatter=create_statement_formatter(version=2),
+            **kwargs,
+        )
 
-    async def _api_request(
+    @check_not_closed
+    async def execute_async(
         self,
-        query: str = "",
-        parameters: Optional[dict[str, Any]] = None,
-        path: str = "",
-        use_set_parameters: bool = True,
-    ) -> Response:
+        query: str,
+        parameters: Optional[Sequence[ParameterType]] = None,
+        skip_parsing: bool = False,
+    ) -> int:
         """
-        Query API, return Response object.
+        Execute a database query without maintating a connection.
+
+        Supported features:
+            Parameterized queries: placeholder characters ('?') are substituted
+                with values provided in `parameters`. Values are formatted to
+                be properly recognized by database and to exclude SQL injection.
+
+        Not supported:
+            Multi-statement queries: multiple statements, provided in a single query
+                and separated by semicolon.
+            SET statements: to provide additional query execution parameters, execute
+                `SET param=value` statement before it. Use `execute` method to set
+                parameters.
 
         Args:
-            query (str): SQL query
+            query (str): SQL query to execute
             parameters (Optional[Sequence[ParameterType]]): A sequence of substitution
                 parameters. Used to replace '?' placeholders inside a query with
-                actual values. Note: In order to "output_format" dict value, it
-                    must be an empty string. If no value not specified,
-                    JSON_OUTPUT_FORMAT will be used.
-            path (str): endpoint suffix, for example "cancel" or "status"
-            use_set_parameters: Optional[bool]: Some queries will fail if additional
-                set parameters are sent. Setting this to False will allow
-                self._set_parameters to be ignored.
+                actual values
+            skip_parsing (bool): Flag to disable query parsing. This will
+                disable parameterized queries while potentially improving performance
+
+        Returns:
+            int: Always returns -1, as async execution does not return row count.
         """
-        parameters = parameters or {}
-        if use_set_parameters:
-            parameters = {**(self._set_parameters or {}), **parameters}
-        if self.parameters:
-            parameters = {**self.parameters, **parameters}
-        if self.connection._is_system:
-            assert isinstance(self._client, AsyncClientV2)
-            parameters["account_id"] = await self._client.account_id
-        return await self._client.request(
-            url=f"/{path}" if path else "",
-            method="POST",
-            params=parameters,
-            content=query,
+        await self._do_execute(
+            query,
+            [parameters] if parameters else [],
+            skip_parsing,
+            async_execution=True,
         )
+        return -1
 
     async def is_db_available(self, database_name: str) -> bool:
         """
@@ -468,17 +643,9 @@ class CursorV2(Cursor):
             connection (firebolt.async_db.connection.Connection)
             database_name (str): Name of a database
         """
-        system_engine = self.connection._system_engine_connection or self.connection
-        with system_engine.cursor() as cursor:
-            return (
-                await cursor.execute(
-                    """
-                    SELECT 1 FROM information_schema.databases WHERE database_name=?
-                    """,
-                    [database_name],
-                )
-                > 0
-            )
+        # For v2 accounts if we're connected it automatically
+        # means the database is available
+        return True
 
     async def is_engine_running(self, engine_url: str) -> bool:
         """
@@ -488,35 +655,9 @@ class CursorV2(Cursor):
             connection (firebolt.async_db.connection.Connection): connection.
             engine_url (str): URL of the engine
         """
-
-        if self.connection._is_system:
-            # System engine is always running
-            return True
-
-        engine_name = URL(engine_url).host.split(".")[0].replace("-", "_")
-        assert self.connection._system_engine_connection is not None  # Type check
-        system_cursor = self.connection._system_engine_connection.cursor()
-        assert isinstance(system_cursor, CursorV2)  # Type check, should always be true
-        (
-            _,
-            status,
-            _,
-        ) = await system_cursor._get_engine_url_status_db(engine_name)
-        return status == ENGINE_STATUS_RUNNING
-
-    async def _get_engine_url_status_db(self, engine_name: str) -> Tuple[str, str, str]:
-        await self.execute(
-            """
-            SELECT url, attached_to, status FROM information_schema.engines
-            WHERE engine_name=?
-            """,
-            [engine_name],
-        )
-        row = await self.fetchone()
-        if row is None:
-            raise FireboltEngineError(f"Engine with name {engine_name} doesn't exist")
-        engine_url, database, status = row
-        return str(engine_url), str(status), str(database)  # Mypy check
+        # For v2 accounts we don't have the engine context,
+        # so we can't check if it's running
+        return True
 
 
 class CursorV1(Cursor):
@@ -528,42 +669,12 @@ class CursorV1(Cursor):
         **kwargs: Any,
     ) -> None:
         assert isinstance(client, AsyncClientV1)
-        super().__init__(*args, client=client, connection=connection, **kwargs)
-
-    async def _api_request(
-        self,
-        query: Optional[str] = "",
-        parameters: Optional[dict[str, Any]] = {},
-        path: Optional[str] = "",
-        use_set_parameters: Optional[bool] = True,
-    ) -> Response:
-        """
-        Query API, return Response object.
-
-        Args:
-            query (str): SQL query
-            parameters (Optional[Sequence[ParameterType]]): A sequence of substitution
-                parameters. Used to replace '?' placeholders inside a query with
-                actual values. Note: In order to "output_format" dict value, it
-                    must be an empty string. If no value not specified,
-                    JSON_OUTPUT_FORMAT will be used.
-            path (str): endpoint suffix, for example "cancel" or "status"
-            use_set_parameters: Optional[bool]: Some queries will fail if additional
-                set parameters are sent. Setting this to False will allow
-                self._set_parameters to be ignored.
-        """
-        parameters = parameters or {}
-        if use_set_parameters:
-            parameters = {**(self._set_parameters or {}), **(parameters or {})}
-        if self.parameters:
-            parameters = {**self.parameters, **parameters}
-        return await self._client.request(
-            url=f"/{path}",
-            method="POST",
-            params={
-                **(parameters or dict()),
-            },
-            content=query,
+        super().__init__(
+            *args,
+            client=client,
+            connection=connection,
+            formatter=create_statement_formatter(version=1),
+            **kwargs,
         )
 
     async def is_db_available(self, database_name: str) -> bool:
@@ -606,3 +717,25 @@ class CursorV1(Cursor):
         )
         resp.raise_for_status()
         return resp
+
+    async def execute_async(
+        self,
+        query: str,
+        parameters: Optional[Sequence[ParameterType]] = None,
+        skip_parsing: bool = False,
+    ) -> int:
+        raise V1NotSupportedError("Async execution")
+
+    async def execute_stream(
+        self,
+        query: str,
+        parameters: Optional[Sequence[ParameterType]] = None,
+        skip_parsing: bool = False,
+    ) -> None:
+        raise V1NotSupportedError("Query result streaming")
+
+    @staticmethod
+    def _get_output_format(is_streaming: bool) -> str:
+        """Get output format."""
+        # Streaming is not supported in v1
+        return JSON_OUTPUT_FORMAT
