@@ -1,6 +1,6 @@
 from typing import Optional
 
-from firebolt.client.auth.base import AuthRequest
+from firebolt.client.auth.base import AuthRequest, FireboltAuthVersion
 from firebolt.client.auth.request_auth_base import _RequestBasedAuth
 from firebolt.utils.token_storage import TokenSecureStorage
 from firebolt.utils.urls import AUTH_SERVICE_ACCOUNT_URL
@@ -33,8 +33,6 @@ class ServiceAccount(_RequestBasedAuth):
         "_user_agent",
     )
 
-    requires_response_body = True
-
     def __init__(
         self,
         client_id: str,
@@ -45,13 +43,31 @@ class ServiceAccount(_RequestBasedAuth):
         self.client_secret = client_secret
         super().__init__(use_token_cache)
 
-    def get_firebolt_version(self) -> int:
+    @property
+    def principal(self) -> str:
+        """Get the principal (client id) associated with the auth.
+
+        Returns:
+            str: client id
+        """
+        return self.client_id
+
+    @property
+    def secret(self) -> str:
+        """Get the secret (client secret) associated with the auth.
+
+        Returns:
+            str: client secret
+        """
+        return self.client_secret
+
+    def get_firebolt_version(self) -> FireboltAuthVersion:
         """Get Firebolt version from auth.
 
         Returns:
-            int: Firebolt version
+            FireboltAuthVersion: V1 for Service Account authentication
         """
-        return 1
+        return FireboltAuthVersion.V1
 
     def copy(self) -> "ServiceAccount":
         """Make another auth object with same credentials.
