@@ -3,6 +3,7 @@ from typing import Generator
 from httpx import Request, Response
 
 from firebolt.client.auth import Auth
+from firebolt.client.auth.base import FireboltAuthVersion
 from firebolt.utils.exception import AuthorizationError
 
 
@@ -23,13 +24,31 @@ class Token(Auth):
         super().__init__(use_token_cache=False)
         self._token = token
 
-    def get_firebolt_version(self) -> int:
+    @property
+    def principal(self) -> str:
+        """Get the principal (placeholder) associated with the auth.
+
+        Returns:
+            str: Principal (placeholder)
+        """
+        return "token_principal"
+
+    @property
+    def secret(self) -> str:
+        """Get the secret (token) associated with the auth.
+
+        Returns:
+            str: Secret, which is the token itself
+        """
+        return self._token or "token"
+
+    def get_firebolt_version(self) -> FireboltAuthVersion:
         """Get Firebolt version from auth.
 
         Returns:
-            int: Firebolt version
+            FireboltAuthVersion: V1 for Token authentication
         """
-        return 1
+        return FireboltAuthVersion.V1
 
     def copy(self) -> "Token":
         """Make another auth object with same credentials.
