@@ -22,12 +22,10 @@ from firebolt.client.http_backend import (
 )
 from firebolt.utils.exception import (
     AccountNotFoundError,
-    AccountNotFoundOrNoAccessError,
     FireboltEngineError,
     InterfaceError,
 )
 from firebolt.utils.urls import (
-    ACCOUNT_BY_NAME_URL,
     ACCOUNT_BY_NAME_URL_V1,
     ACCOUNT_ENGINE_ID_BY_NAME_URL,
     ACCOUNT_ENGINE_URL,
@@ -92,19 +90,40 @@ class FireboltClientMixin(FireboltClientMixinBase):
         return request
 
     def _enforce_trailing_slash(self, url: URL) -> URL:
-        """Don't automatically append trailing slach to a base url"""
+        """Don't automatically append trailing slash to a base url"""
         return url
 
+    def clone(self) -> "Client":
+        return self.__class__(
+            auth=self.auth,
+            account_name=self.account_name,
+            base_url=self.base_url,
+            api_endpoint=str(self._api_endpoint),
+            timeout=self.timeout,
+            headers=self.headers,
+        )
+
 
 class Client(FireboltClientMixin, HttpxClient, metaclass=ABCMeta):
     def __init__(self, *args: Any, **kwargs: Any):
-        super().__init__(*args, **kwargs, transport=KeepaliveTransport())
+        super().__init__(
+            *args,
+            **kwargs,
+            transport=KeepaliveTransport(verify=kwargs.get("verify", True)),
+        )
 
-    @cached_property
+    @property
     @abstractmethod
     def account_id(self) -> str:
         ...
 
+    def _send_handling_redirects(
+        self, request: Request, *args: Any, **kwargs: Any
+    ) -> Response:
+        return super()._send_handling_redirects(
+            self._merge_auth_request(request), *args, **kwargs
+        )
+
 
 class ClientV2(Client):
     """An HTTP client, based on httpx.Client.
@@ -130,7 +149,7 @@ class ClientV2(Client):
             **kwargs,
         )
 
-    @cached_property
+    @property
     def account_id(self) -> str:
         """User account ID.
 
@@ -143,24 +162,7 @@ class ClientV2(Client):
         Raises:
             AccountNotFoundError: No account found with provided name
         """
-        response = self.get(
-            url=self._api_endpoint.copy_with(
-                path=ACCOUNT_BY_NAME_URL.format(account_name=self.account_name)
-            )
-        )
-        if response.status_code == HttpxCodes.NOT_FOUND:
-            assert self.account_name is not None
-            raise AccountNotFoundOrNoAccessError(self.account_name)
-        # process all other status codes
-        response.raise_for_status()
-        return response.json()["id"]
-
-    def _send_handling_redirects(
-        self, request: Request, *args: Any, **kwargs: Any
-    ) -> Response:
-        return super()._send_handling_redirects(
-            self._merge_auth_request(request), *args, **kwargs
-        )
+        return ""
 
 
 class ClientV1(Client):
@@ -268,23 +270,27 @@ class ClientV1(Client):
                 f"Unable to retrieve engine endpoint {url}."
             )
 
-    def _send_handling_redirects(
-        self, request: Request, *args: Any, **kwargs: Any
-    ) -> Response:
-        return super()._send_handling_redirects(
-            self._merge_auth_request(request), *args, **kwargs
-        )
-
 
 class AsyncClient(FireboltClientMixin, HttpxAsyncClient, metaclass=ABCMeta):
     def __init__(self, *args: Any, **kwargs: Any):
-        super().__init__(*args, **kwargs, transport=AsyncKeepaliveTransport())
+        super().__init__(
+            *args,
+            **kwargs,
+            transport=AsyncKeepaliveTransport(verify=kwargs.get("verify", True)),
+        )
 
     @property
     @abstractmethod
     async def account_id(self) -> str:
         ...
 
+    async def _send_handling_redirects(
+        self, request: Request, *args: Any, **kwargs: Any
+    ) -> Response:
+        return await super()._send_handling_redirects(
+            self._merge_auth_request(request), *args, **kwargs
+        )
+
 
 class AsyncClientV2(AsyncClient):
     """An HTTP client, based on httpx.Client.
@@ -309,7 +315,6 @@ class AsyncClientV2(AsyncClient):
             api_endpoint=api_endpoint,
             **kwargs,
         )
-        self.acount_id_cache: Dict[str, str] = {}
 
     @property
     async def account_id(self) -> str:
@@ -324,32 +329,7 @@ class AsyncClientV2(AsyncClient):
         Raises:
             AccountNotFoundError: No account found with provided name
         """
-        # manual caching to avoid async_cached_property issues
-        if self.account_name in self.acount_id_cache:
-            return self.acount_id_cache[self.account_name]
-
-        response = await self.get(
-            url=self._api_endpoint.copy_with(
-                path=ACCOUNT_BY_NAME_URL.format(account_name=self.account_name)
-            )
-        )
-        if response.status_code == HttpxCodes.NOT_FOUND:
-            assert self.account_name is not None
-            raise AccountNotFoundOrNoAccessError(self.account_name)
-        # process all other status codes
-        response.raise_for_status()
-        account_id = response.json()["id"]
-        # cache for future use
-        if self.account_name:
-            self.acount_id_cache[self.account_name] = account_id
-        return account_id
-
-    async def _send_handling_redirects(
-        self, request: Request, *args: Any, **kwargs: Any
-    ) -> Response:
-        return await super()._send_handling_redirects(
-            self._merge_auth_request(request), *args, **kwargs
-        )
+        return ""
 
 
 class AsyncClientV1(AsyncClient):
@@ -375,7 +355,7 @@ class AsyncClientV1(AsyncClient):
             api_endpoint=api_endpoint,
             **kwargs,
         )
-        self.acount_id_cache: Dict[str, str] = {}
+        self.account_id_cache: Dict[str, str] = {}
         self._auth_endpoint = URL(fix_url_schema(api_endpoint))
 
     @property
@@ -392,8 +372,8 @@ class AsyncClientV1(AsyncClient):
             AccountNotFoundError: No account found with provided name
         """
         # manual caching to avoid async_cached_property issues
-        if self.account_name in self.acount_id_cache:
-            return self.acount_id_cache[self.account_name]
+        if self.account_name in self.account_id_cache:
+            return self.account_id_cache[self.account_name]
 
         if self.account_name:
             response = await self.get(
@@ -411,7 +391,7 @@ class AsyncClientV1(AsyncClient):
             self.account_name = account_response["name"]
             assert self.account_name is not None  # type check
         # cache for future use
-        self.acount_id_cache[self.account_name] = account_id
+        self.account_id_cache[self.account_name] = account_id
         return account_id
 
     async def _get_database_default_engine_url(
@@ -467,10 +447,3 @@ class AsyncClientV1(AsyncClient):
                 f"Error {e.__class__.__name__}: "
                 f"Unable to retrieve engine endpoint {url}."
             )
-
-    async def _send_handling_redirects(
-        self, request: Request, *args: Any, **kwargs: Any
-    ) -> Response:
-        return await super()._send_handling_redirects(
-            self._merge_auth_request(request), *args, **kwargs
-        )
