@@ -1,6 +1,5 @@
 from typing import TYPE_CHECKING
 
-from firebolt.client import ClientV2
 from firebolt.db import Connection
 
 if TYPE_CHECKING:
@@ -11,14 +10,6 @@ class BaseService:
     def __init__(self, resource_manager: "ResourceManager"):
         self.resource_manager = resource_manager
 
-    @property
-    def client(self) -> ClientV2:
-        return self.resource_manager._client
-
-    @property
-    def account_id(self) -> str:
-        return self.resource_manager.account_id
-
     @property
     def _connection(self) -> Connection:
         return self.resource_manager._connection
