@@ -1,3 +1,6 @@
+from typing import Any, Dict, Optional
+
+
 class FireboltError(Exception):
     """Base class for all Firebolt errors."""
 
@@ -176,17 +179,22 @@ class AuthenticationError(FireboltError):
 
 
 class AuthorizationError(FireboltError):
-    """Firebolt authentication error.
+    """Firebolt authorisation error.
 
     Args:
         cause (str): Reason for authorization failure
 
-    Attributes:
-        cause (str): Reason for authorization failure
     """
 
-    def __init__(self, cause: str):
-        super().__init__(f"Authorization failed: {cause}.")
+    _default_error_message = (
+        "Failed to connect to Firebolt. Could not authenticate with the given "
+        "credentials. Please verify the provided credentials are up to date and "
+        "correct and that you have the correct user permissions"
+    )
+
+    def __init__(self, cause: Optional[str] = None):
+        error_cause = cause if cause else self._default_error_message
+        super().__init__(f"Authorization failed: {error_cause}.")
 
 
 # PEP-249
@@ -265,16 +273,82 @@ class NotSupportedError(DatabaseError):
     """
 
 
+class V1NotSupportedError(NotSupportedError):
+    """Operation not supported in Firebolt V1
+
+    Exception raised when trying to use the functionality
+    that is not supported in Firebolt V1.
+    """
+
+    msg = (
+        "{} is not supported in this version of Firebolt. "
+        "Please contact support to upgrade your account to a new version."
+    )
+
+    def __init__(self, operation: str) -> None:
+
+        super().__init__(self.msg.format(operation))
+
+
 class ConfigurationError(InterfaceError):
     """Invalid configuration error."""
 
 
-class AsyncExecutionUnavailableError(ProgrammingError):
-    """
-    If `use_standard_sql` is specified the query status endpoint returns a JSON
-    object with empty values instead of a proper status object. In that case,
-    it is not possible to retrieve the results of an asynchronous query.
+class FireboltStructuredError(ProgrammingError):
+    """Base class for structured errors received in JSON body."""
+
+    # Output will look like this after formatting:
+    # "{severity}: {name} ({code}) - {description}, see {helpLink}"
+    message_template = "{severity}{name}{code}{description}{at}{helpLink}"
+
+    def __init__(self, json: Dict[str, Any]):
+        self.json = json
+        self.errors = json.get("errors", [])
+
+    def __str__(self) -> str:
+        error_messages = []
+        for error in self.errors:
+            severity = f"{error['severity']}: " if error.get("severity") else ""
+            name = f"{error['name']} " if error.get("name") else ""
+            code = f"({error['code']}) " if error.get("code") else ""
+            description = (
+                f"- {error['description']}" if error.get("description") else ""
+            )
+            helpLink = (  # NOSONAR: python:S608 compliant with the message template
+                f", see {error['helpLink']}" if error.get("helpLink") else ""
+            )
+            at = f" at {error['location']}" if error.get("location") else ""
+            message = self.message_template.format(
+                severity=severity,
+                name=name,
+                code=code,
+                description=description,
+                helpLink=helpLink,
+                at=at,
+            )
+            error_messages.append(message)
+        return ",\n".join(error_messages)
+
+
+class QueryTimeoutError(FireboltError, TimeoutError):
+    """Query execution timed out.
+
+    Exception raised when the query execution exceeds the specified timeout.
     """
 
-    def __init__(self, error_message):  # type: ignore
-        super().__init__(error_message)
+    def __init__(self, message="Query execution timed out."):  # type: ignore
+        super().__init__(message)
+
+
+class MethodNotAllowedInAsyncError(FireboltError):
+    """Method not allowed.
+
+    Exception raised when the method is not allowed.
+    """
+
+    def __init__(self, method_name: str):
+        super().__init__(
+            f"Method {method_name} not allowed for an async query."
+            " Please get the token and use the async query API to get the status."
+        )
+        self.method_name = method_name
