@@ -3,11 +3,24 @@ from functools import lru_cache
 from os import environ
 from time import time
 from types import TracebackType
-from typing import TYPE_CHECKING, Callable, Optional, Type, TypeVar
+from typing import (
+    TYPE_CHECKING,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+)
+from urllib.parse import parse_qs, urljoin, urlparse
 
 from httpx import URL, Response, codes
 
-from firebolt.utils.exception import ConfigurationError
+from firebolt.utils.exception import (
+    ConfigurationError,
+    FireboltStructuredError,
+)
 
 T = TypeVar("T")
 logger = logging.getLogger(__name__)
@@ -147,17 +160,33 @@ def validate_engine_name_and_url_v1(
         )
 
 
-def _print_error_body(resp: Response) -> None:
-    """log error body if it exists, since it's not always logged by default"""
+def raise_error_from_response(resp: Response) -> None:
+    """
+    Raise a correct error from the response.
+    Look for a structured error in the body and raise it.
+    If the body doesn't contain a structured error,
+    log the body and raise a status code error.
+
+    Args:
+        resp (Response): HTTP response
+    """
+    to_raise = None
     try:
-        if (
-            codes.is_error(resp.status_code)
-            and "Content-Length" in resp.headers
-            and int(resp.headers["Content-Length"]) > 0
-        ):
-            logger.error(f"Something went wrong: {resp.read().decode('utf-8')}")
+        decoded = resp.json()
+        if "errors" in decoded and len(decoded["errors"]) > 0:
+            # Raise later to avoid catching it in the except block
+            to_raise = FireboltStructuredError(decoded)
+
     except Exception:
-        pass
+        # If we can't parse the body, print out the error body
+        if "Content-Length" in resp.headers and int(resp.headers["Content-Length"]) > 0:
+            logger.error(f"Something went wrong: {resp.read().decode('utf-8')}")
+
+    if to_raise:
+        raise to_raise
+
+    # Raise status error if no error info was found in the body
+    resp.raise_for_status()
 
 
 class Timer:
@@ -181,3 +210,41 @@ class Timer:
         ):
             log_message = self._message + self.elapsed_time + "s"
             logger.debug(log_message)
+
+
+def parse_url_and_params(url: str) -> Tuple[str, Dict[str, str]]:
+    """Extract URL and query parameters separately from a URL."""
+    url = fix_url_schema(url)
+    parsed_url = urlparse(url)
+    query_params = parse_qs(parsed_url.query)
+    # This strips query parameters from the URL by joining base URL and path
+    # skipping the query parameters.
+    result_url = urljoin(url, parsed_url.path)
+    # parse_qs returns a dictionary with values as lists.
+    # We want the last value in the list.
+    query_params_dict = {}
+    for key, values in query_params.items():
+        # Multiple values for the same key are not expected
+        if len(values) > 1:
+            raise ValueError(f"Multiple values found for key '{key}'")
+        query_params_dict[key] = values[0]
+    return result_url, query_params_dict
+
+
+class _ExceptionGroup(Exception):
+    """A base class for grouping exceptions.
+
+    This class is used to create an exception group that can contain multiple
+    exceptions. It is a placeholder for Python 3.11's ExceptionGroup, which
+    allows for grouping exceptions together.
+    """
+
+    def __init__(self, message: str, exceptions: List[BaseException]):
+        super().__init__(message)
+        self.exceptions = exceptions
+
+    def __str__(self) -> str:
+        return f"{self.__class__.__name__}({self.exceptions})"
+
+
+ExceptionGroup = getattr(__builtins__, "ExceptionGroup", _ExceptionGroup)
